'use strict';

var webpack = require('webpack');

/**
 * Get configuration for Webpack
 *
 * @see http://webpack.github.io/docs/configuration
 *      https://github.com/petehunt/webpack-howto
 *
 * @param {boolean} release True if configuration is intended to be used in
 * a release mode, false otherwise
 * @return {object} Webpack configuration
 */
module.exports = function(release) {
    return {
        entry: './src/app.js',

        output: {
            filename: 'app.js',
            path: './build/js/',
            publicPath: './build/js/'
        },

        cache: !release,
        debug: !release,
        devtool: false,

        stats: {
            colors: true,
            reasons: !release
        },

        plugins: release ? [
            new webpack.DefinePlugin({
                'process.env.NODE_ENV': '"production"',
                '__DEV__': false
            }),
            new webpack.optimize.DedupePlugin(),
            new webpack.optimize.UglifyJsPlugin(),
            new webpack.optimize.OccurenceOrderPlugin(),
            new webpack.optimize.AggressiveMergingPlugin()
        ] : [
            new webpack.DefinePlugin({'__DEV__': true})
        ],

        resolve: {
            extensions: ['', '.webpack.js', '.web.js', '.js']
        },

        module: {
            loaders: [
                {
                    test: /\.css$/,
                    loader: 'style-loader!css-loader!autoprefixer-loader?{browsers:[' +
                        '"Android 2.3", "Android >= 4", "Chrome >= 20", "Firefox >= 24", ' +
                        '"Explorer >= 8", "iOS >= 6", "Opera >= 12", "Safari >= 6"]}'
                },
                {
                    test: /\.less$/,
                    loader: 'style-loader!css-loader!autoprefixer-loader?{browsers:[' +
                        '"Android 2.3", "Android >= 4", "Chrome >= 20", "Firefox >= 24", ' +
                        '"Explorer >= 8", "iOS >= 6", "Opera >= 12", "Safari >= 6"]}!less-loader'
                },
                {
                    test: /\.gif/,
                    loader: 'url-loader?limit=10000&mimetype=image/gif'
                },
                {
                    test: /\.jpg/,
                    loader: 'url-loader?limit=10000&mimetype=image/jpg'
                },
                {
                    test: /\.png/,
                    loader: 'url-loader?limit=10000&mimetype=image/png'
                },
                {
                    test: /\.js?$/,
                    loader: 'jsx-loader?harmony&stripTypes'
                }
            ]
        }
    };
};
