'use strict';

var g = require('gulp'),
    p = require('gulp-load-plugins')(),
    argv = require('minimist')(process.argv.slice(2)),
    webpack = require('webpack'),
    runSequence = require('run-sequence'),
    del = require('del'),
    stylish = require('jshint-stylish');

var RELEASE = !!argv.release,
    AUTOPREFIXER_BROWSERS = [
        'ie >= 10',
        'ie_mob >= 10',
        'ff >= 30',
        'chrome >= 34',
        'safari >= 7',
        'opera >= 23',
        'ios >= 7',
        'android >= 4.4',
        'bb >= 10'
    ];

var watch = false,
    paths = {
        js: 'src/**/*.js',
        images: 'src/images/*',
        pages: 'src/pages/*.html'
    };

var src = {};

g.task('clean', function(cb) {
    return del(['build'], cb);
});

g.task('vendor', function() {
    return g
        .src('src/vendor/*.js')
        .pipe(g.dest('build/js'))
        .pipe(p.size({title: 'vendor'}));
});

g.task('vendor-bootstrap', function() {
    return g
        .src('node_modules/bootstrap/dist/fonts/**')
        .pipe(g.dest('build/fonts'))
        .pipe(p.size({title: 'vendor-bootstrap'}));
});

g.task('bundle', function(cb) {
    var started = false,
        config = require('./config/webpack.js')(RELEASE),
        bundler = webpack(config);

    function bundle(err, stats) {
        if (err) {
            throw new p.util.PluginError('webpack', err);
        }

        if (argv.verbose) {
            p.util.log('[webpack]', stats.toString({colors: true}));
        }

        if (!started) {
            started = true;
            return cb();
        }

        return false;
    }

    if (watch) {
        bundler.watch(200, bundle);
    } else {
        bundler.run(bundle);
    }
});

g.task('jshint', function() {
    var stream = g
            .src(['./src/**/**/*.js', 'gulpfile.js'])
            .pipe(p.insert.prepend('/** @jsx React.DOM */'))
            .pipe(p.react({harmony: true, stripTypes: true}))
            .pipe(p.jshint())
            .pipe(p.jshint.reporter(stylish));

    if (process.env.CI) {
        stream = stream.pipe(p.jshint.reporter('fail'));
    }

    return stream;
});

g.task('styles', function() {
    src.styles = ('src/styles/**/*.{css,less}');

    return g.src('src/styles/bootstrap.less')
        .pipe(p.plumber())
        .pipe(p.less({
            sourceMap: !RELEASE,
            sourceMapBasepath: __dirname
        }))
        .on('error', console.error.bind(console))
        .pipe(p.autoprefixer({
            browsers: AUTOPREFIXER_BROWSERS
        }))
        .pipe(p.csscomb())
        .pipe(p.if(RELEASE, p.minifyCss()))
        .pipe(g.dest('build/css'))
        .pipe(p.size({title: 'styles'}));
});

g.task('images', function() {
    return g
        .src(paths.images)
        .pipe(g.dest('build/images'))
        .pipe(p.size({title: 'images'}));
});

g.task('pages', function() {
    return g
        .src(paths.pages)
        .pipe(g.dest('build'))
        .pipe(p.size({title: 'pages'}));
});

g.task('watch', function(cb) {
    watch = true;

    g.watch(paths.styles, ['styles']);

    runSequence('bundle');

    cb();

    console.log('Watching files for changes...');
});

g.task('build', function(cb) {
    runSequence('clean', ['styles', 'bundle', 'vendor', 'vendor-bootstrap', 'images', 'pages'], cb);
});

g.task('default', ['build']);
