'use strict';

var ClientAudit = require('./clients/audit'),
    ClientGeneric = require('./clients/generic'),
    ClientList = require('./clients/list'),
    ClientSearch = require('./clients/search'),
    ClientMembers = require('./clients/members'),
    ClientLayerMembers = require('./clients/layer_members'),
    ClientGraph = require('./clients/graph');

var constants = require('./constants');

module.exports = {
    flashMessage(msg, status) {
        this.dispatch(constants.FLASH_MESSAGE, {msg: msg, status: status});
    },

    routes: {
        transition(params) {
            this.dispatch(constants.ROUTES.TRANSITION, {params: params});
        }
    },

    search(query) {
        this.dispatch(constants.SEARCH.LOAD, {id: query});

        ClientSearch(
            query,
            (res) => {
                var payload = {id: query, type: 'SEARCH_RESULT', res: res};

                this.dispatch(
                    constants.SEARCH.LOAD_SUCCESS,
                    payload
                );
            },
            (res) => {
                this.dispatch(constants.SEARCH.LOAD_FAIL, {id: query, type: 'SEARCH_RESULT'});
                this.flux.actions.flashMessage(res.error.message);
            }
        );
    },

    audit: {
        loadAll(startFlow) {
            this.dispatch(constants.AUDIT.LOAD);

            ClientAudit.fetch(
                ClientAudit.c.AUDIT_ALL,
                null,
                null,
                (res) => {
                    var payload = {res: res, startFlow: startFlow};

                    this.dispatch(
                        constants.AUDIT.LOAD_SUCCESS,
                        payload
                    );
                },
                (res) => {
                    this.dispatch(constants.AUDIT.LOAD_FAIL);
                    this.flux.actions.flashMessage(res.error.message);
                }
            );
        },

        loadFlow(lastEventId) {
            ClientAudit.fetch(
                ClientAudit.c.AUDIT_FLOW,
                null,
                null,
                (res) => {
                    var event = JSON.parse(res.data);

                    if (event.id > lastEventId) {
                        var payload = {event: event};

                        this.dispatch(
                            constants.AUDIT.LOAD_SUCCESS,
                            payload
                        );
                    }
                },
                (res) => {
                    // this.dispatch(constants.AUDIT.LOAD_FAIL);
                    // this.flux.actions.flashMessage(res.error.message);
                }
            );
        },

        loadForRole(kind, id) {
            this.dispatch(constants.AUDIT.LOAD);

            ClientAudit.fetch(
                ClientAudit.c.AUDIT_ROLE,
                kind,
                id,
                (res) => {
                    var payload = {res: res};

                    this.dispatch(
                        constants.AUDIT.LOAD_SUCCESS,
                        payload
                    );
                },
                (res) => {
                    this.dispatch(constants.AUDIT.LOAD_FAIL, {kind: kind, id: id});
                    this.flux.actions.flashMessage(res.error.message);
                });
        },

        loadForResource(kind, id) {
            this.dispatch(constants.AUDIT.LOAD);

            ClientAudit.fetch(
                ClientAudit.c.AUDIT_RESOURCE,
                kind,
                id,
                (res) => {
                    var payload = {res: res};

                    this.dispatch(
                        constants.AUDIT.LOAD_SUCCESS,
                        payload
                    );
                },
                (res) => {
                    this.dispatch(constants.AUDIT.LOAD_FAIL, {kind: kind, id: id});
                    this.flux.actions.flashMessage(res.error.message);
                });
        }
    },

    resources: {
        load(kind) {
            this.dispatch(constants.RESOURCES.LOAD);

            ClientList.fetch(kind, (res) => {
                this.dispatch(constants.RESOURCES.LOAD_SUCCESS, {res: res});
            }, (err) => {
                this.dispatch(constants.RESOURCES.LOAD_FAIL, {id: kind});
                this.flux.actions.flashMessage(err.error.message);
            });
        },

        loadOne(kind, id) {
            this.dispatch(constants.RESOURCE.LOAD);

            ClientList.fetchOne(kind, id, (res) => {
                this.dispatch(constants.RESOURCE.LOAD_SUCCESS, {kind: kind, id: id, res: res});
            }, (err) => {
                this.dispatch(constants.RESOURCE.LOAD_FAIL, {kind: kind, id: id});
                this.flux.actions.flashMessage(err.error.message);
            });
        }
    },

    graph: {
        load(kind, id) {
            this.dispatch(constants.GRAPH.LOAD);

            // this.dispatch(constants.GRAPH.LOAD_SUCCESS,
            //               {kind: kind, id: id,
            //                res: {body: {"graph":[{"parent":"conjur:group:foods","child":"conjur:host:ldap1"},{"parent":"conjur:group:vegetables","child":"conjur:host:ldap1"},{"parent":"conjur:user:foo","child":"conjur:host:ldap1"},{"parent":"conjur:user:bar","child":"conjur:host:ldap1"},{"parent":"conjur:group:foods","child":"conjur:group:vegetables"},{"parent":"conjur:host:ldap1","child":"conjur:user:admin"}]}}});

            ClientGraph(
                kind,
                id,
                (res) => {
                    var payload = {kind: kind, id: id, res: res};

                    this.dispatch(
                        constants.GRAPH.LOAD_SUCCESS,
                        payload
                    );
                },
                (res) => {
                    this.dispatch(constants.GRAPH.LOAD_FAIL, {id: id});
                    // this.flux.actions.flashMessage(res.error.message);
                }
            );
        }
    },

    user: {
        load(id) {
            this.dispatch(constants.USER.LOAD, {id: id});

            [
                ClientGeneric.c.ATTRIBUTES,
                ClientGeneric.c.OWNED_RESOURCES,
                ClientGeneric.c.ALL_ROLES,
                ClientGeneric.c.PUBLIC_KEYS
            ].forEach((type) => {
                this.dispatch(constants.USER.LOAD, {id: id, type: type});

                ClientGeneric.fetch(
                    type,
                    'user',
                    id,
                    (res) => {
                        var payload = {id: id, type: type, res: res};

                        this.dispatch(
                            constants.USER.LOAD_SUCCESS,
                            payload
                        );
                    },
                    (res) => {
                        this.dispatch(constants.USER.LOAD_FAIL, {type: type});
                        this.flux.actions.flashMessage(res.error.message);
                    });
            });

            this.dispatch(constants.USER.LOAD, {id: id, type: 'RESOURCES'});

            ClientList.fetchResources(
                'user',
                id,
                (res) => {
                    var payload = {id: id, type: 'RESOURCES', res: res};

                    this.dispatch(
                        constants.USER.LOAD_SUCCESS,
                        payload
                    );
                },
                (res) => {
                    this.dispatch(constants.USER.LOAD_FAIL, {type: 'RESOURCES'});
                    this.flux.actions.flashMessage(res.error.message);
                });
        }
    },

    group: {
        load(id) {
            this.dispatch(constants.GROUP.LOAD, {id: id});

            [
                ClientGeneric.c.ATTRIBUTES,
                ClientGeneric.c.OWNED_RESOURCES,
                ClientGeneric.c.ALL_ROLES
            ].forEach((type) => {
                this.dispatch(constants.GROUP.LOAD, {id: id, type: type});

                ClientGeneric.fetch(
                    type,
                    'group',
                    id,
                    (res) => {
                        var payload = {id: id, type: type, res: res};

                        this.dispatch(
                            constants.GROUP.LOAD_SUCCESS,
                            payload
                        );
                    },
                    (res) => {
                        this.dispatch(constants.GROUP.LOAD_FAIL, {type: type});
                        this.flux.actions.flashMessage(res.error.message);
                    });
            });

            this.dispatch(constants.GROUP.LOAD, {id: id, type: 'GROUP_ROLES'});

            ClientList.fetchGroupRoles(
                id,
                (res) => {
                    var payload = {id: id, type: 'GROUP_ROLES', res: res};

                    this.dispatch(
                        constants.GROUP.LOAD_SUCCESS,
                        payload
                    );
                },
                (res) => {
                    this.dispatch(constants.GROUP.LOAD_FAIL, {type: 'GROUP_ROLES'});
                    this.flux.actions.flashMessage(res.error.message);
                });

            this.dispatch(constants.GROUP.LOAD, {id: id, type: 'RESOURCES'});

            ClientList.fetchResources(
                'group',
                id,
                (res) => {
                    var payload = {id: id, type: 'RESOURCES', res: res};

                    this.dispatch(
                        constants.GROUP.LOAD_SUCCESS,
                        payload
                    );
                },
                (res) => {
                    this.dispatch(constants.GROUP.LOAD_FAIL, {type: 'RESOURCES'});
                    this.flux.actions.flashMessage(res.error.message);
                });
        }
    },

    host: {
        load(id) {
            this.dispatch(constants.HOST.LOAD, {id: id});

            [
                ClientGeneric.c.ATTRIBUTES,
                ClientGeneric.c.OWNED_RESOURCES,
                ClientGeneric.c.ALL_ROLES
            ].forEach((type) => {
                this.dispatch(constants.HOST.LOAD, {id: id, type: type});

                ClientGeneric.fetch(
                    type,
                    'host',
                    id,
                    (res) => {
                        var payload = {id: id, type: type, res: res};

                        this.dispatch(
                            constants.HOST.LOAD_SUCCESS,
                            payload
                        );
                    },
                    (res) => {
                        this.dispatch(constants.HOST.LOAD_FAIL, {type: type});
                        this.flux.actions.flashMessage(res.error.message);
                    });
            });

            [
                ClientMembers.c.MEMBERS_EXECUTE,
                ClientMembers.c.MEMBERS_UPDATE
            ].forEach((type) => {
                this.dispatch(constants.HOST.LOAD, {id: id, type: type});

                ClientMembers.fetch(
                    constants.HOST.ASSET_TYPE,
                    type,
                    id,
                    (res) => {
                        var payload = {id: id, type: type, res: res};

                        this.dispatch(
                            constants.HOST.LOAD_SUCCESS,
                            payload
                        );
                    },
                    (res) => {
                        this.dispatch(constants.HOST.LOAD_FAIL, {type: type});
                        this.flux.actions.flashMessage(res.error.message);
                    });
            });

            this.dispatch(constants.HOST.LOAD, {id: id, type: 'RESOURCES'});

            ClientList.fetchResources(
                'host',
                id,
                (res) => {
                    var payload = {id: id, type: 'RESOURCES', res: res};

                    this.dispatch(
                        constants.HOST.LOAD_SUCCESS,
                        payload
                    );
                },
                (res) => {
                    this.dispatch(constants.HOST.LOAD_FAIL, {type: 'RESOURCES'});
                    this.flux.actions.flashMessage(res.error.message);
                });
        }
    },

    layer: {
        load(id) {
            this.dispatch(constants.LAYER.LOAD, {id: id});

            [
                ClientGeneric.c.ATTRIBUTES,
                ClientGeneric.c.OWNED_RESOURCES,
                ClientGeneric.c.ALL_ROLES
            ].forEach((type) => {
                this.dispatch(constants.LAYER.LOAD, {id: id, type: type});

                ClientGeneric.fetch(
                    type,
                    'layer',
                    id,
                    (res) => {
                        var payload = {id: id, type: type, res: res};

                        this.dispatch(
                            constants.LAYER.LOAD_SUCCESS,
                            payload
                        );
                    },
                    (res) => {
                        this.dispatch(constants.LAYER.LOAD_FAIL, {type: type});
                        this.flux.actions.flashMessage(res.error.message);
                    });
            });

            [
                ClientLayerMembers.c.LAYER_MEMBERS_USE,
                ClientLayerMembers.c.LAYER_MEMBERS_ADMIN
            ].forEach((type) => {
                this.dispatch(constants.LAYER.LOAD, {id: id, type: type});

                ClientLayerMembers.fetch(
                    type,
                    id,
                    (res) => {
                        var payload = {id: id, type: type, res: res};

                        this.dispatch(
                            constants.LAYER.LOAD_SUCCESS,
                            payload
                        );
                    },
                    (res) => {
                        this.dispatch(constants.LAYER.LOAD_FAIL, {type: type});
                        this.flux.actions.flashMessage(res.error.message);
                    });
            });

            this.dispatch(constants.LAYER.LOAD, {id: id, type: 'RESOURCES'});

            ClientList.fetchResources(
                'layer',
                id,
                (res) => {
                    var payload = {id: id, type: 'RESOURCES', res: res};

                    this.dispatch(
                        constants.LAYER.LOAD_SUCCESS,
                        payload
                    );
                },
                (res) => {
                    this.dispatch(constants.LAYER.LOAD_FAIL, {type: 'RESOURCES'});
                    this.flux.actions.flashMessage(res.error.message);
                });
        }
    },

    variable: {
        load(id) {
            this.dispatch(constants.VARIABLE.LOAD, {id: id});

            [
                ClientGeneric.c.ATTRIBUTES
            ].forEach((type) => {
                this.dispatch(constants.VARIABLE.LOAD, {id: id, type: type});

                ClientGeneric.fetch(
                    type,
                    'variable',
                    id,
                    (res) => {
                        var payload = {id: id, type: type, res: res};

                        this.dispatch(
                            constants.VARIABLE.LOAD_SUCCESS,
                            payload
                        );
                    },
                    (res) => {
                        this.dispatch(constants.VARIABLE.LOAD_FAIL, {type: type});
                        this.flux.actions.flashMessage(res.error.message);
                    });
            });

            [
                ClientMembers.c.MEMBERS_EXECUTE,
                ClientMembers.c.MEMBERS_UPDATE
            ].forEach((type) => {
                this.dispatch(constants.VARIABLE.LOAD, {id: id, type: type});

                ClientMembers.fetch(
                    constants.VARIABLE.ASSET_TYPE,
                    type,
                    id,
                    (res) => {
                        var payload = {id: id, type: type, res: res};

                        this.dispatch(
                            constants.VARIABLE.LOAD_SUCCESS,
                            payload
                        );
                    },
                    (res) => {
                        this.dispatch(constants.VARIABLE.LOAD_FAIL, {type: type});
                        this.flux.actions.flashMessage(res.error.message);
                    });
            });
        }
    },

    policy: {
        load(id) {
            this.dispatch(constants.POLICY.LOAD, {id: id});

            [
                ClientGeneric.c.OWNED_RESOURCES,
                ClientGeneric.c.ALL_ROLES
            ].forEach((type) => {
                this.dispatch(constants.POLICY.LOAD, {id: id, type: type});

                ClientGeneric.fetch(
                    type,
                    'policy',
                    id,
                    (res) => {
                        var payload = {id: id, type: type, res: res};

                        this.dispatch(
                            constants.POLICY.LOAD_SUCCESS,
                            payload
                        );
                    },
                    (res) => {
                        this.dispatch(constants.POLICY.LOAD_FAIL, {type: type});
                        this.flux.actions.flashMessage(res.error.message);
                    });
            });

            this.dispatch(constants.POLICY.LOAD, {id: id, type: 'RESOURCES'});

            ClientList.fetchResources(
                'policy',
                id,
                (res) => {
                    var payload = {id: id, type: 'RESOURCES', res: res};

                    this.dispatch(
                        constants.POLICY.LOAD_SUCCESS,
                        payload
                    );
                },
                (res) => {
                    this.dispatch(constants.POLICY.LOAD_FAIL, {type: 'RESOURCES'});
                    this.flux.actions.flashMessage(res.error.message);
                });
        }
    }
};
