'use strict';

var utils = require('./utils');

var conjur = window.conjur = {
    app: {
        configuration: undefined,
        userId: undefined
    }
};

var cookie = utils.readCookie('conjur_configuration');

conjur.app.configuration = JSON.parse(decodeURIComponent(cookie).replace(/\+/g, ' '));
conjur.app.userId = window.decodeURIComponent(utils.readCookie('conjur_userid'));

var React = require('react'),
    Router = require('react-router'),
    Fluxxor = require('fluxxor');

var actions = require('./actions'),
    routes = require('./routes');

var AppStore = require('./stores/app_store'),
    RouteStore = require('./stores/route_store'),
    SearchStore = require('./stores/search_store'),
    AuditStore = require('./stores/audit_store'),
    ResourcesStore = require('./stores/resources_store'),
    GraphStore = require('./stores/graph_store');

var UserStore = require('./stores/user_store'),
    GroupStore = require('./stores/group_store'),
    HostStore = require('./stores/host_store'),
    LayerStore = require('./stores/layer_store'),
    VariableStore = require('./stores/variable_store'),
    PolicyStore = require('./stores/policy_store');

var router = Router.create({routes: routes, location: Router.HistoryLocation});

var stores = {
    AppStore: new AppStore(),
    route: new RouteStore(),
    search: new SearchStore(),
    audit: new AuditStore(),
    resources: new ResourcesStore(),
    graph: new GraphStore(),
    user: new UserStore(),
    group: new GroupStore(),
    host: new HostStore(),
    layer: new LayerStore(),
    variable: new VariableStore(),
    policy: new PolicyStore()
};

var flux = new Fluxxor.Flux(stores, actions);

flux.on('dispatch', function(type, payload) {
    if (console && console.log) {
        console.log('[Dispatch]', type, payload);
    }
});

Promise.all([
    new Promise((resolve) => {
        if (window.addEventListener) {
            window.addEventListener('DOMContentLoaded', resolve);
        } else {
            window.attachEvent('onload', resolve);
        }
    })
]).then(() => {
    router.run(function(Handler, state) {
        flux.actions.routes.transition(state.params);

        React.render(
            <Handler flux={flux} />,
            document.getElementById('wrapper')
        );
    });
});
