/* global conjur */

'use strict';

var request = require('superagent');

var utils = require('../utils');

var c = module.exports.c = {
    ATTRIBUTES: 'ATTRIBUTES',
    OWNED_RESOURCES: 'OWNED_RESOURCES',
    ALL_ROLES: 'ALL_ROLES',
    RESOURCE: 'RESOURCE',
    PUBLIC_KEYS: 'PUBLIC_KEYS'
};

module.exports.fetch = function(type, kind, id, callback, errCallback) {
    var defaultCallback = function(res) {
        if (res.error) {
            errCallback(res);
        } else {
            callback(res);
        }
    };

    switch (type) {
    case c.ATTRIBUTES:
        request
            .get('/api/'+
                 utils.pluralize(kind) +
                 '/' +
                 window.encodeURIComponent(id))
            .end(defaultCallback);
        break;

    case c.OWNED_RESOURCES:
        var roleid = [kind, id].join(':');

        request
            .get('/api/authz/' +
                 conjur.app.configuration.account +
                 '/resources?owner=' +
                 roleid)
            .end(function(res) {
                if (res.error) {
                    errCallback(res);
                } else {
                    res.body = res.body.filter(function(item) {
                        // assets of 'secret' kind are internal
                        return item.id.split(':')[1] !== 'secret';
                    });

                    callback(res);
                }
            });
        break;

    case c.ALL_ROLES:
        request
            .get('/api/authz/' +
                 conjur.app.configuration.account +
                 '/roles/' +
                 window.encodeURIComponent(kind) +
                 '/' +
                 window.encodeURIComponent(id) +
                 '?all')
            .end(defaultCallback);
        break;

    case c.RESOURCE:
        request
            .get('/api/authz/' +
                 conjur.app.configuration.account +
                 '/resources/' +
                 kind +
                 '/' +
                 window.encodeURIComponent(id))
            .end(defaultCallback);
        break;

    case c.PUBLIC_KEYS:
        request
            .get('/api/pubkeys/' + window.encodeURIComponent(id))
            .end(defaultCallback);
        break;
    };
};
