'use strict';

var React = require('react'),
    update = require('react/lib/update'),
    sortByOrder = require('lodash/collection/sortByOrder'),
    includes = require('lodash/collection/includes'),
    isString = require('lodash/lang/isString');

var AuditEntry = require('./entry'),
    AuditTableHeader = require('./table_header'),
    Refresh = require('../refresh/refresh'),
    constants = require('./constants');

module.exports = React.createClass({
    displayName: 'AuditTable',

    propTypes: {
        caption: React.PropTypes.string,
        compact: React.PropTypes.bool,
        events: React.PropTypes.array,
        isLoading: React.PropTypes.bool
    },

    getDefaultProps() {
        return {
            caption: 'Audit',
            compact: false,
            events: [],
            isLoading: false
        };
    },

    render() {
        var entries = sortByOrder(this.filterEvents(), ['id'], [false]).map((event) => {
            // new way to clone objects, custom clone$ does not work no more as expected
            var args = update(event, {
                $merge: {
                    key: event.id,
                    compact: this.props.compact
                }
            });

            return (
              <AuditEntry {...args} />
            );
        });

        var title;

        if (this.props.caption !== '') {
            title = (
              <h3>{this.props.caption}<Refresh show={this.props.isLoading} /></h3>
            );
        }

        return (
          <section className="audit">
            {title}
            <table className="audit-table">
              <AuditTableHeader key="thead" compact={this.props.compact} />
              <tbody key="tbody">
                {entries}
              </tbody>
            </table>
          </section>
        );
    },

    filterEvents() {
        return this.props.events.filter(function(ev) {
            // remove internal resources creation
            if (isString(ev.role) && (ev.role.split(':')[1] === '@')) {
                return false;
            }

            if (isString(ev.resource) && (ev.resource.split(':')[1] === '@')) {
                return false;
            }

            if (isString(ev.resource) && (ev.resource.split(':')[1] === 'secret')) {
                return false;
            }

            if (isString(ev.grantee) && (ev.grantee.split(':')[1] === '@')) {
                return false;
            }

            // hide automated creation of roles corresponding to resources of known type
            if ((ev.kind === 'role') && (ev.action === 'create')) {
                var roleKind = ev.role.split(':')[1];

                if (includes(constants.knownRolsourceTypes, roleKind)) {
                    return false;
                }
            }

            // hide new event role:index / all_roles , which is used by role graph only
            if ((ev.kind === 'role') && (ev.action === 'index')) {
                return false;
            }

            if ((ev.kind === 'role') && (ev.action === 'all_roles')) {
                return false;
            }

            return true;
        });
    }
});
