'use strict';

var React = require('react'),
    filter = require('lodash/collection/filter'),
    take = require('lodash/array/take'),
    map = require('lodash/collection/map'),
    includes = require('lodash/collection/includes'),
    isString = require('lodash/lang/isString');

var defaultNumberOfElements = 3;

// TODO: refactor to avoid this copypaste from audit.js
var knownRolsourceTypes = ['user', 'group', 'host', 'layer', 'policy'];

var AuditEntry = require('../audit/entry'),
    ResourceLink = require('./resource_link'),
    Time = require('./time'),
    Refresh = require('../refresh/refresh');

var FoldableAuditSection = React.createClass({
    displayName: 'FoldableAuditSection',

    statics: {
        updates(audit, isLoading) {
            isLoading = isLoading || false;

            var filter = function(e) {
                return (
                    e.action === 'check' && e.privilege === 'update' &&
                        !(e.hasOwnProperty('error') ||
                          e.allowed === false)
                );
            };

            var getItem = function(e) {
                var user = [
                    (<Time timestamp={e.timestamp} relative="time from now" />),
                    (<ResourceLink id={e.user} />)
                ];

                if (e.user !== e.acting_as) {
                    user.push(' acting as ');
                    user.push(<ResourceLink id={e.acting_as} />);
                }

                return user;
            };

            return (
                <FoldableAuditSection audit={audit} title="Updates"
                                      filter={filter} getItem={getItem}
                                      openMsg="See full updates history"
                                      isLoading={isLoading} />
            );
        },

        warnings(audit, isLoading) {
            isLoading = isLoading || false;

            var filter = function(e) {
                // hide new event role:index / all_roles , which is used by role graph only
                if ((e.kind === 'role') && (e.action === 'index')) {
                    return false;
                }

                if ((e.kind === 'role') && (e.action === 'all_roles')) {
                    return false;
                }

                return (
                    e.hasOwnProperty('error') || e.allowed === false
                );
            };

            var getItem = function(e) {
                return AuditEntry(e);
            };

            return (
              <FoldableAuditSection audit={audit} title="Warnings"
                                    filter={filter} getItem={getItem}
                                    openMsg="See full warnings history"
                                    isLoading={isLoading} />
            );
        },

        changes(audit, isLoading) {
            isLoading = isLoading || false;

            var filter = function(e) {
                // hide new event role:index / all_roles , which is used by role graph only
                if ((e.kind === 'role') && (e.action === 'index')) {
                    return false;
                }

                if ((e.kind === 'role') && (e.action === 'all_roles')) {
                    return false;
                }

                if (e.action === 'check') {
                    return false;
                }

                if (e.kind === 'audit' || e.kind === 'annotation') {
                    return false;
                }

                // TODO: refactor code to avoid copypaste from audit.js
                // remove internal resources creation
                if (isString(e.role) && (e.role.split(':')[1] === '@')) {
                    return false;
                }

                if (isString(e.resource) && (e.resource.split(':')[1] === '@')) {
                    return false;
                }

                if (isString(e.resource) && (e.resource.split(':')[1] === 'secret')) {
                    return false;
                }

                if (isString(e.grantee) && (e.grantee.split(':')[1] === '@')) {
                    return false;
                }

                // hide automated creation of roles corresponding to resources of known type
                if ((e.kind === 'role') && (e.action === 'create')) {
                    var rolekind = e.role.split(':')[1];

                    if (includes(knownRolsourceTypes, rolekind)) {
                        return false;
                    }
                }

                // anything else is permission change
                return true;
            };

            var getItem = function(e) {
                return AuditEntry(e);
            };

            return (
              <FoldableAuditSection audit={audit} title="Recent permission model changes"
                                    filter={filter} getItem={getItem}
                                    openMsg="See full history of permission model changes"
                                    isLoading={isLoading} />
            );
        }
    },

    propTypes: {
        audit: React.PropTypes.arrayOf(React.PropTypes.object),
        filter: React.PropTypes.func,
        getItem: React.PropTypes.func,
        howMuch: React.PropTypes.number,
        title: React.PropTypes.string,
        openMsg: React.PropTypes.string,
        isLoading: React.PropTypes.bool
    },

    defaultFilter(e) {
        return typeof e === 'object';
    },

    defaultGetItem(e) {
        return e.id;
    },

    getDefaultProps() {
        return {
            audit: [],
            filter: this.defaultFilter,
            getItem: this.defaultGetItem,
            howMuch: defaultNumberOfElements,
            title: 'Audit Section',
            openMsg: 'See full history',
            isLoading: false
        };
    },

    getInitialState() {
        return {showAll: false};
    },

    handleClick(e) {
        e.preventDefault();

        this.setState({showAll: !this.state.showAll});
    },

    reverseChronology(a, b) {
        if (a.timestamp < b.timestamp) {
            return 1;
        } else if (a.timestamp > b.timestamp) {
            return -1;
        } else {
            return 0;
        }
    },

    getItems() {
        var items = filter(this.props.audit,
                           this.props.filter)
                .sort(this.reverseChronology),
            className = [
                'b-audit-section__item',
                'list-group-item',
                'list-group-item-noborder'
            ].join(' ');

        if (!this.state.showAll) {
            items = take(items, this.props.howMuch);
        }

        return map(items, function(e) {
            return (
              <li className={className}
                  key={'audit-entry-' + e.id}>
                {this.props.getItem(e)}
              </li>
            );
        }.bind(this));
    },

    render() {
        var items = this.getItems(),
            msg = this.state.showAll ?
                'Show only top ' + (this.props.howMuch) :
                this.props.openMsg,
            empty = items.length === 0,
            lessThan = items.length < (this.props.howMuch),
            body = [
              <h2 key="title">
                {this.props.title}<Refresh show={this.props.isLoading} />
              </h2>
            ];

        if (!empty) {
            body.push(
              <ul className="b-audit-section__list list-unstyled list-group"
                  key="list">
                {items}
              </ul>
            );
        } else {
            body.push(
              <p key="info">There is no history records available</p>
            );
        }

        if (!lessThan) {
            body.push(
              <div className="b-audit-section__toggle"
                   key="toggle">
                <a href="#" onClick={this.handleClick}>{msg}</a>
              </div>
            );
        }

        return (
          <div className="b-audit-section">
            {body}
          </div>
        );
    }
});

module.exports = FoldableAuditSection;
