'use strict';

var _ = require('lodash');

var Fluxxor = require('fluxxor'),
    SortedSet = require("collections/sorted-set");

var c = require('../constants'),
    utils = require('../utils');

module.exports = Fluxxor.createStore({
    initialize() {
        this.loading = 0;
        this.events = SortedSet([], function(a, b) {
            return b.id === a.id;
        }, function(a, b) {
            if (a.id < b.id) {
                return -1;
            }

            if (a.id > b.id) {
                return 1;
            }

            return 0;
        });

        this.bindActions(
            c.AUDIT.LOAD, this.onLoad,
            c.AUDIT.LOAD_SUCCESS, this.onLoadSuccess,
            c.AUDIT.LOAD_FAIL, this.onLoadFail
        );
    },

    onLoad() {
        this.loading = this.loading + 1;
        this.emit('change');
    },

    onLoadSuccess(payload) {
        if (typeof payload.event === 'object') {
            this.events.push(payload.event);
            this.emit('change');
        } else {
            this.loading = this.loading - 1 < 0 ? 0 : this.loading - 1;
            this.events.addEach(payload.res.body);
            this.emit('change');
        }

        if (payload.startFlow === true) {
            this.flux.actions.audit.loadFlow(this.events.findGreatest().value.id);
        }
    },

    onLoadFail(payload) {
        this.loading = this.loading - 1 < 0 ? 0 : this.loading - 1;
        this.emit('change');
    },

    getAllEvents() {
        return {
            loading: Boolean(this.loading),
            data: this.events.toArray()
        };
    },

    getEventFor(kind, id) {
        var fullId = conjur.app.configuration.account + ':' + kind + ':' + id;

        return {
            loading: Boolean(this.loading),
            data: this.events.filter(function(event) {
                return event.role === fullId || event.resource === fullId;
            }).toArray()
        };
    }
});
