'use strict';

var Fluxxor = require('fluxxor');

var ClientGeneric = require('../clients/generic'),
    constants = require('../constants').USER,
    utils = require('../utils');

var fieldMapping = {
    'ATTRIBUTES': 'user',
    'OWNED_RESOURCES': 'owned',
    'ALL_ROLES': 'roles',
    'PUBLIC_KEYS': 'pubkeys',
    'RESOURCES': 'resources'
};

module.exports = Fluxxor.createStore({
    initialize() {
        this.data = this._fresh();

        this.bindActions(
            constants.LOAD, this.onLoad,
            constants.LOAD_SUCCESS, this.onLoadSuccess,
            constants.LOAD_FAIL, this.onLoadFail
        );
    },

    _fresh() {
        return {
            userId: null,
            user: {
                login: '',
                roleId: ''
            },
            pubkeys: [],
            loading: {
                user: true,
                owned: true,
                roles: true,
                pubkeys: true,
                resources: true
            }
        };
    },

    onLoad(payload) {
        if (typeof payload.type === 'undefined') {
            this.data = this._fresh();
            this.data.identifier = payload.id;
            this.emit('change');
        } else if (payload.id === this.data.identifier) {
            this.turn(payload.type, true);
            this.emit('change');
        }
    },

    onLoadSuccess(payload) {
        if (payload.id === this.data.identifier) {
            this.turn(payload.type, false);

            this.setData(payload);
            this.emit('change');
        }
    },

    onLoadFail(payload) {
        this.turn(payload.type, false);
    },

    turn(type, state) {
        var fields = fieldMapping[type];

        if (typeof fields === 'object') {
            fields.forEach((field) => {
                this.data.loading[field] = state;
            });
        } else {
            this.data.loading[fields] = state;
        }
    },

    setData(payload) {
        switch (payload.type) {
        case ClientGeneric.c.ATTRIBUTES:
            this.data.user = payload.res.body;
            break;

        case ClientGeneric.c.OWNED_RESOURCES:
            this.data.owned = payload.res.body;
            break;

        case ClientGeneric.c.ALL_ROLES:
            this.data.roles = payload.res.body;
            break;

        case ClientGeneric.c.PUBLIC_KEYS:
            this.data.pubkeys = payload.res.text.split('\n').filter(function(d) {
                return d.trim().length > 0;
            });
            break;

        case 'RESOURCES':
            this.data.resources = payload.res.body;
            break;
        }
    },

    getData() {
        return this.data;
    }
});
