'use strict';

var Fluxxor = require('fluxxor');

var ClientGeneric = require('../clients/generic'),
    ClientMembers = require('../clients/members'),
    constants = require('../constants').VARIABLE;

var fieldMapping = {
    'ATTRIBUTES': 'variable',
    'MEMBERS_EXECUTE': 'fetchers',
    'MEMBERS_UPDATE': 'updaters'
};

module.exports = Fluxxor.createStore({
    initialize() {
        this.data = this._fresh();

        this.bindActions(
            constants.LOAD, this.onLoad,
            constants.LOAD_SUCCESS, this.onLoadSuccess,
            constants.LOAD_FAIL, this.onLoadFail
        );
    },

    _fresh() {
        return {
            variable: {
                userid: ''
            },
            loading: {
                variable: true,
                fetchers: true,
                updaters: true
            }
        };
    },

    onLoad(payload) {
        if (typeof payload.type === 'undefined') {
            this.data = this._fresh();
            this.data.identifier = payload.id;
            this.emit('change');
        } else if (payload.id === this.data.identifier) {
            this.turn(payload.type, true);
            this.emit('change');
        }
    },

    onLoadSuccess(payload) {
        if (payload.id === this.data.identifier) {
            this.turn(payload.type, false);

            this.setData(payload);
            this.emit('change');
        }
    },

    onLoadFail(payload) {
        this.turn(payload.type, false);
    },

    turn(type, state) {
        var fields = fieldMapping[type];

        if (typeof fields === 'object') {
            fields.forEach((field) => {
                this.data.loading[field] = state;
            });
        } else {
            this.data.loading[fields] = state;
        }
    },

    setData(payload) {
        switch (payload.type) {
        case ClientGeneric.c.ATTRIBUTES:
            this.data.variable = payload.res.body;
            break;

        case ClientMembers.c.MEMBERS_EXECUTE:
            this.data.fetchers = payload.res.body;
            break;

        case ClientMembers.c.MEMBERS_UPDATE:
            this.data.updaters = payload.res.body;
            break;
        }
    },

    getData() {
        return this.data;
    }
});
